@@ -1,3 +1,4 @@
+import os
 import copy
 from pathlib import Path
 from typing import Awaitable, Callable
@@ -16,6 +17,12 @@ _AiohttpView = Callable[[web.Request], Awaitable[web.StreamResponse]]
 
 VALID_RESPONSE_FIELDS = {"description", "headers", "examples"}
 
+NAME_SWAGGER_SPEC = "swagger.spec"
+NAME_SWAGGER_DOCS = "swagger.docs"
+NAME_SWAGGER_STATIC = "swagger.static"
+
+INDEX_PAGE = "index.html"
+
 
 def resolver(schema):
     schema_instance = common.resolve_schema_instance(schema)
@@ -38,10 +45,11 @@ class AiohttpApiSpec:
         error_callback=None,
         in_place=False,
         prefix='',
+        schema_name_resolver=resolver,
         **kwargs
     ):
 
-        self.plugin = MarshmallowPlugin(schema_name_resolver=resolver)
+        self.plugin = MarshmallowPlugin(schema_name_resolver=schema_name_resolver)
         self.spec = APISpec(plugins=(self.plugin,), openapi_version="2.0", **kwargs)
 
         self.url = url
@@ -51,6 +59,7 @@ class AiohttpApiSpec:
         self._request_data_name = request_data_name
         self.error_callback = error_callback
         self.prefix = prefix
+        self._index_page = None
         if app is not None:
             self.register(app, in_place)
 
@@ -84,24 +93,40 @@ class AiohttpApiSpec:
             async def swagger_handler(request):
                 return web.json_response(request.app["swagger_dict"])
 
-            app.router.add_route("GET", self.url, swagger_handler, name="swagger.spec")
+            route_url = self.url
+            if not self.url.startswith("/"):
+                route_url = "/{}".format(self.url)
+            app.router.add_route("GET", route_url, swagger_handler, name=NAME_SWAGGER_SPEC)
 
             if self.swagger_path is not None:
                 self._add_swagger_web_page(app, self.static_path, self.swagger_path)
 
+    def _get_index_page(self, app, static_files, static_path):
+        if self._index_page is not None:
+            return self._index_page
+
+        with open(str(static_files / INDEX_PAGE)) as swg_tmp:
+            url = self.url if app is None else app.router[NAME_SWAGGER_SPEC].url_for()
+
+            if app is not None:
+                static_path = app.router[NAME_SWAGGER_STATIC].url_for(filename=INDEX_PAGE)
+                static_path = os.path.dirname(str(static_path))
+
+            self._index_page = Template(swg_tmp.read()).render(path=url, static=static_path)
+
+        return self._index_page
+
     def _add_swagger_web_page(
         self, app: web.Application, static_path: str, view_path: str
     ):
         static_files = Path(__file__).parent / "static"
-        app.router.add_static(static_path, static_files)
-
-        with open(str(static_files / "index.html")) as swg_tmp:
-            tmp = Template(swg_tmp.read()).render(path=self.url, static=static_path)
+        app.router.add_static(static_path, static_files, name=NAME_SWAGGER_STATIC)
 
         async def swagger_view(_):
-            return web.Response(text=tmp, content_type="text/html")
+            index_page = self._get_index_page(app, static_files, static_path)
+            return web.Response(text=index_page, content_type="text/html")
 
-        app.router.add_route("GET", view_path, swagger_view, name="swagger.docs")
+        app.router.add_route("GET", view_path, swagger_view, name=NAME_SWAGGER_DOCS)
 
     def _register(self, app: web.Application):
         for route in app.router.routes():
@@ -137,6 +162,7 @@ class AiohttpApiSpec:
             parameters = self.plugin.converter.schema2parameters(
                 schema["schema"], **schema["options"]
             )
+            self._add_examples(schema["schema"], parameters, schema["example"])
             data["parameters"].extend(parameters)
 
         existing = [p["name"] for p in data["parameters"] if p["in"] == "path"]
@@ -171,6 +197,24 @@ class AiohttpApiSpec:
         operations = copy.deepcopy(data)
         self.spec.path(path=url_path, operations={method: operations})
 
+    def _add_examples(self, ref_schema, endpoint_schema, example):
+        def add_to_endpoint_or_ref():
+            if add_to_refs:
+                self.spec.components._schemas[name]["example"] = example
+            else:
+                endpoint_schema[0]['schema']['allOf'] = [endpoint_schema[0]['schema'].pop('$ref')]
+                endpoint_schema[0]['schema']["example"] = example
+        if not example:
+            return
+        schema_instance = common.resolve_schema_instance(ref_schema)
+        name = self.plugin.converter.schema_name_resolver(schema_instance)
+        add_to_refs = example.pop('add_to_refs')
+        if self.spec.components.openapi_version.major < 3:
+            if name and name in self.spec.components._schemas:
+                add_to_endpoint_or_ref()
+        else:
+            add_to_endpoint_or_ref()
+
 
 def setup_aiohttp_apispec(
     app: web.Application,
@@ -184,6 +228,7 @@ def setup_aiohttp_apispec(
     error_callback=None,
     in_place: bool = False,
     prefix: str = '',
+    schema_name_resolver: Callable = resolver,
     **kwargs
 ) -> None:
     """
@@ -240,6 +285,7 @@ def setup_aiohttp_apispec(
                      instead of the moment of the on_startup signal.
                      If True, be sure all routes are added to router
     :param prefix: prefix to add to all registered routes
+    :param schema_name_resolver: custom schema_name_resolver for MarshmallowPlugin.
     :param kwargs: any apispec.APISpec kwargs
     """
     AiohttpApiSpec(
@@ -253,5 +299,6 @@ def setup_aiohttp_apispec(
         error_callback=error_callback,
         in_place=in_place,
         prefix=prefix,
+        schema_name_resolver=schema_name_resolver,
         **kwargs
     )
